# -*- coding: iso-8859-15 -*-

# Contractor, a contract validation tool
#    Copyright (C) 2008  Guido de Caso, Víctor Braberman, Diego Garbervetsky, Sebastián Uchitel
#    {gdecaso, vbraber, diegog, suchitel}@dc.uba.ar
#    LaFHIS - Departamento de Computación - Facultad de Ciencias Exactas y Naturales - Universidad de Buenos Aires
#    Pabellón I - Ciudad Universitaria - (C1428EGA) - Buenos Aires - Argentina
#    Tel: +54-11-4576-3390 eext. 704
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys

def graphviz_write_fsm(fsm, contract, options):
	
	# write header
	print "# autogenerated with con2fsm" # TODO tool name
	print "# transitions that may lead to a broken contract invariant are marked with an '*'"
	print ""
	print "digraph " + contract.name + " {"
	print "\trankdir=LR;"
		
	# write initial state (if any)
	if fsm.initial_state != None:
		print "\tnode [shape = doublecircle", 
		if not options.print_state_numbers:
			print ", label=\"\"",
		print "]; S" + str(fsm.initial_state) + ";"
		print "\tnode [shape = circle];"
		
		# calculate a mapping that stores every action that goes from s1 to s2
		# (so we can use a single arrow with a label that shows many actions)
		actions = {}
		for state in fsm.states:
			for transition in state.transitions:
				if state.name not in actions:
					state_actions = {}
					state_actions[transition.destination] = [transition]
					actions[state.name] = state_actions
				else:
					if transition.destination not in actions[state.name]:
						actions[state.name][transition.destination] = [transition]
					else:
						actions[state.name][transition.destination].append(transition)

		# write each transition
		for state in actions:
			for destination in actions[state]:
				print "\tS" + str(state) + " -> S" + str(destination) + " [ label= \"",
				transition_number = 1
				for transition in actions[state][destination]:
					label = transition.action
					if transition.violates_invariant:
						label += "*"
					if transition_number < len(actions[state][destination]):
						label += options.separator
					sys.stdout.write(label)
					transition_number += 1
				print "\" ];"
	print "}"
